/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static void action_button(GtkWidget *widget, gpointer data)
{
	GtkWidget *mainform;
	const char *name;
	const char *mainform_name;
	char *parent_name = NULL;
	int keyboard_state = 0;

	g_assert(GTK_IS_BUTTON(widget));
	name = get_object_relative_name(GTK_OBJECT(widget));
	g_return_if_fail(name != NULL);

	/* Disable all windows while Linuxconf processes. */
	window_list_busy_all(NULL, NULL);

	/* Get the name of the mainform we're in. */
	mainform = (GtkWidget*) get_object_mainform(GTK_OBJECT(widget));
	g_return_if_fail(GTK_IS_WIDGET(mainform));
	mainform_name = get_object_base_name(GTK_OBJECT(mainform));

	/* Dump widgets of the subform. */
	if(gtk_object_get_data(GTK_OBJECT(widget), GNOME_LINUXCONF_NEEDS_DUMP)){
		dump_object(NULL,mainform);
	}

	/* Get the parent widget's name. */
	parent_name = g_strdup(name);
	if(strrchr(parent_name, '.') != NULL) {
		*(strrchr(parent_name, '.')) = '\0';
	}
	/* Tell the back-end we were clicked. */
#ifdef DEBUG
	printf("action %s %s %d\n", strstr(parent_name, mainform_name),
	       get_object_base_name(GTK_OBJECT(widget)), keyboard_state);
#endif
	g_print("action %s %s %d\n", strstr(parent_name, mainform_name),
		get_object_base_name(GTK_OBJECT(widget)), keyboard_state);

	g_free(parent_name);
}

/* A regular button. */
DUMP_DEF(Button)
{
}

HANDLE_DEF(Button)
{
	GtkWidget *widget = NULL;
	char *id = (char*)g_list_nth_data(args, 0);
	char *dump = g_list_nth_data(args, 1);
	char *title = (char*)g_list_nth_data(args, 2);

	/* If title is not there, it's because dump was omitted. */
	if(title == NULL) {
		title = dump;
		dump = NULL;
	}

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(title != NULL, NULL);

	/* Create a new label for the widget text. */
	title = g_strdup(title);
	widget = gtk_button_new_with_label(title);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			   GTK_SIGNAL_FUNC(action_button), NULL);
	gtk_signal_connect(GTK_OBJECT(widget), "destroy",
			   GTK_SIGNAL_FUNC(free_data), title);
	gtk_misc_set_padding(GTK_MISC((GTK_BIN(widget))->child), GNOME_PAD, 0);

	if((dump != NULL) && (atoi(dump) != 0)) {
		gtk_object_set_data(GTK_OBJECT(widget),
				    GNOME_LINUXCONF_NEEDS_DUMP,
				    GNOME_LINUXCONF_NEEDS_DUMP);
	}

        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_HFLAGS,
                            GINT_TO_POINTER(GTK_EXPAND | GTK_FILL));
        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VFLAGS,
                            GINT_TO_POINTER(GTK_SHRINK));
        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VPAD,
                            GINT_TO_POINTER(GNOME_PAD_SMALL));

	parent_insert(NULL, GTK_OBJECT(widget));

	register_object_with_parent(GTK_OBJECT(widget), id);

	return GTK_OBJECT(widget);
}

/* A button with a graphic in it. */
DUMP_DEF(Button_xpm)
{
}

HANDLE_DEF(Button_xpm)
{
	GtkWidget *widget = NULL, *pm, *vbox;
	char *id = (char*)g_list_nth_data(args, 0);
	char *icon_name = (char*)g_list_nth_data(args, 1);
	char *title = (char*)g_list_nth_data(args, 2);
	GdkPixmap *pixmap;
	GdkBitmap *bitmap;

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(icon_name != NULL, NULL);

	pm = (GtkWidget*) get_xpm_object_by_name(icon_name);
	g_return_val_if_fail(GTK_IS_PIXMAP(pm), NULL);
	gtk_pixmap_get(GTK_PIXMAP(pm),&pixmap,&bitmap);
	pm = gtk_pixmap_new(pixmap,bitmap);
	
	widget = gtk_button_new();
	vbox = gtk_vbox_new(FALSE,5);
	gtk_container_add(GTK_CONTAINER(widget),vbox);
	gtk_box_pack_start(GTK_BOX(vbox),pm,FALSE,FALSE,0);
	
	if(title != NULL) {
		GtkWidget *label = gtk_label_new(title);
		gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	}
	
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			   GTK_SIGNAL_FUNC(action_button), NULL);

        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_HFLAGS,
                            GINT_TO_POINTER(GTK_FILL));
        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VFLAGS,
                            GINT_TO_POINTER(GTK_SHRINK));
        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VPAD,
                            GINT_TO_POINTER(GNOME_PAD_SMALL));

	parent_insert(NULL, GTK_OBJECT(widget));

	register_object_with_parent(GTK_OBJECT(widget), id);

	return GTK_OBJECT(widget);
}

/* A button with a graphic (given by a filename) in it. */
DUMP_DEF(Button_xpmf)
{
}

HANDLE_DEF(Button_xpmf)
{
	GtkWidget *widget = NULL, *pm, *vbox;
	char *id = (char*)g_list_nth_data(args, 0);
	char *icon_name = (char*) g_list_nth_data(args, 1);
	char *title = (char*) g_list_nth_data(args, 2);
	GdkPixmap *pixmap;
	GdkBitmap *bitmap;

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(icon_name != NULL, NULL);

	pixmap = gdk_pixmap_colormap_create_from_xpm(NULL,
			gtk_widget_get_default_colormap(),&bitmap,NULL,
			icon_name);
	pm = gtk_pixmap_new(pixmap,bitmap);
	
	widget = gtk_button_new();
	vbox = gtk_vbox_new(FALSE,5);
	gtk_container_add(GTK_CONTAINER(widget),vbox);
	gtk_box_pack_start(GTK_BOX(vbox),pm,FALSE,FALSE,0);
	
	if(title != NULL) {
		GtkWidget *label = gtk_label_new(title);
		gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	}
	
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			   GTK_SIGNAL_FUNC(action_button), NULL);

        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_HFLAGS,
                            GINT_TO_POINTER(GTK_FILL));
        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VFLAGS,
                            GINT_TO_POINTER(GTK_SHRINK));
        gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VPAD,
                            GINT_TO_POINTER(GNOME_PAD_SMALL));

	parent_insert(NULL, GTK_OBJECT(widget));

	register_object_with_parent(GTK_OBJECT(widget), id);

	return GTK_OBJECT(widget);
}

DUMP_DEF(Buttonfill)
{
}

HANDLE_DEF(Buttonfill)
{
	GtkWidget *widget = NULL;
	char *id = (char*)g_list_nth_data(args, 0);
	char *dump = g_list_nth_data(args, 1);
	char *title = (char*)g_list_nth_data(args, 2);

	/* If title is not there, it's because dump was omitted. */
	if(title == NULL) {
		title = dump;
		dump = NULL;
	}

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(title != NULL, NULL);

	/* Create a new label for the widget text. */
	title = g_strdup(title);
	widget = gtk_button_new_with_label(title);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			   GTK_SIGNAL_FUNC(action_button), NULL);
	gtk_signal_connect(GTK_OBJECT(widget), "destroy",
			   GTK_SIGNAL_FUNC(free_data), title);
	gtk_misc_set_padding(GTK_MISC((GTK_BIN(widget))->child), GNOME_PAD, 0);

	if((dump != NULL) && (atoi(dump) != 0)) {
		gtk_object_set_data(GTK_OBJECT(widget),
				    GNOME_LINUXCONF_NEEDS_DUMP,
				    GNOME_LINUXCONF_NEEDS_DUMP);
	}

	gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VFLAGS,
			    GINT_TO_POINTER(GTK_SHRINK));
	gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_VPAD,
			    GINT_TO_POINTER(GNOME_PAD_SMALL));
	gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_HINT_HPAD,
			    GINT_TO_POINTER(GNOME_PAD_SMALL));
	parent_insert(NULL, GTK_OBJECT(widget));

	register_object_with_parent(GTK_OBJECT(widget), id);

	return GTK_OBJECT(widget);
}

SETVAL_DEF(Button)
{
        const char *pid = (const char*) g_list_nth_data(args, 0);
        const char *id = (const char*) g_list_nth_data(args, 1);
	const char *label = (const char*) g_list_nth_data(args, 2);
	GtkObject *object;
	g_return_if_fail(label != NULL);
	object = get_object_by_name_with_parent(strlen(pid)>0?pid:NULL,id);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(object)->child), label);
}

SETVAL_DEF(Button_xpm)
{
        const char *pid = (const char*) g_list_nth_data(args, 0);
        const char *id = (const char*) g_list_nth_data(args, 1);
	const char *icon_name = (const char*) g_list_nth_data(args, 2);
	const char *title = (const char*) g_list_nth_data(args, 3);
	GtkObject *object;
	GtkWidget *pm, *vbox;
	GdkPixmap *pixmap;
	GdkBitmap *bitmap;
	
	g_return_if_fail(icon_name != NULL);

	object = get_object_by_name_with_parent(strlen(pid)>0?pid:NULL,id);
	
	gtk_widget_destroy(GTK_BIN(object)->child);
	
	pm = (GtkWidget*) get_xpm_object_by_name(icon_name);
	g_return_if_fail(GTK_IS_PIXMAP(pm));
	gtk_pixmap_get(GTK_PIXMAP(pm),&pixmap,&bitmap);
	pm = gtk_pixmap_new(pixmap,bitmap);
	
	vbox = gtk_vbox_new(FALSE,5);
	gtk_container_add(GTK_CONTAINER(object),vbox);
	gtk_box_pack_start(GTK_BOX(vbox),pm,FALSE,FALSE,0);
	
	if(title != NULL) {
		GtkWidget *label = gtk_label_new(title);
		gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	}

	gtk_widget_show_all(vbox);
}
