/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static void choice_destroy_item(GtkObject *item, gpointer data)
{
	GtkCList *clist = GTK_CLIST(GTK_DOUBLE_COMBO(GTK_GENERIC(item)->parent)->clist);
	GtkEntry *entry = GTK_ENTRY(GTK_DOUBLE_COMBO(GTK_GENERIC(item)->parent)->entry);
	guint dh = GPOINTER_TO_UINT(gtk_object_get_data(item,
					GNOME_LINUXCONF_DESTROY_HANDLER));
	char *entrytext;
	int row = clist_find_row(clist, get_object_base_name(item));
	g_return_if_fail(row != -1);
	entrytext = g_strdup(gtk_entry_get_text(entry));
	gtk_clist_remove(clist, row);
	gtk_entry_set_text(entry,entrytext);
	g_free(entrytext);
	gtk_signal_disconnect(GTK_OBJECT(GTK_GENERIC(item)->parent), dh);
}

DUMP_DEF(Choice)
{
	GtkWidget *entry;
	const char *text;
	const char *cname = get_object_relative_name(dumpee);
       	char *name, *c;

	g_return_if_fail(cname != NULL);
	name = g_strdup(cname);
	c = strrchr(name, '.');

	if(c != NULL) {
		*c++ = '\0';
		entry = (GTK_DOUBLE_COMBO(dumpee))->entry;
		text = gtk_entry_get_text(GTK_ENTRY(entry));
		if((text != NULL) && (strlen(text) != 0)) {
#ifdef DEBUG
			printf("dump %s %s %s\n", name, c, text);
#endif
			g_print("dump %s %s %s\n", name, c, text);
		}
	}

	g_free(name);
}

HANDLE_DEF(Choice)
{
	GtkWidget *entry;
	GtkObject *choice;
	char *id = (char*)g_list_nth_data(args, 0);
	char *val = (char*)g_list_nth_data(args, 1);

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(val != NULL, NULL);
	
	choice = (GtkObject*) gtk_double_combo_new();
	gtk_double_combo_disable_activate(GTK_DOUBLE_COMBO(choice));

	entry = (GTK_DOUBLE_COMBO(choice))->entry;
	gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
	gtk_entry_set_text(GTK_ENTRY(entry), val);

	val = g_strdup(val);
	gtk_object_set_data(GTK_OBJECT(entry), GNOME_LINUXCONF_COMBO_DEFAULT,
			    val);
	gtk_signal_connect(GTK_OBJECT(entry), "destroy",
			   GTK_SIGNAL_FUNC(free_data), val);
	
	gtk_object_set_data(choice, GNOME_LINUXCONF_HINT_VFLAGS,
			    GINT_TO_POINTER(GTK_FILL));
	gtk_signal_connect(parents_peek(), "dump",
			   GTK_SIGNAL_FUNC(dump_object), choice);
	
	parent_insert(NULL, choice);
	parents_push(choice);
	register_object_with_parent(choice, id);

	return choice;
}

HANDLE_DEF(Choice_item)
{
	GtkWidget *entry;
	GtkObject *choice, *item;
	GtkCList *clist;
	const char *choiceid = (char*)g_list_nth_data(args,0);
	char *id = (char*)g_list_nth_data(args, 1);
	char *val = (char*)g_list_nth_data(args, 2);
	char *comments = concat_strings(g_list_next(g_list_next(g_list_next(args))));
	char *entrytext;
	char *array[2];
	char *did;
	int row;
	uint handler;

	g_return_val_if_fail(choiceid != NULL, NULL);
	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(val != NULL, NULL);

	if (strlen(choiceid) == 0) {
		choice = parents_peek();
		choiceid = get_object_name(choice);
	} else {
		choice = get_object_by_name(choiceid);
	}
	
	g_return_val_if_fail(GTK_IS_DOUBLE_COMBO(choice), NULL);

	entry = (GTK_DOUBLE_COMBO(choice))->entry;
	entrytext = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

	array[0] = val;
	array[1] = comments;
	did = g_strdup(id);
	clist = GTK_CLIST(GTK_DOUBLE_COMBO(choice)->clist);
	row = gtk_clist_append(clist, array);
	gtk_clist_set_row_data(clist, row, did);
	gtk_clist_columns_autosize(clist);
	gtk_entry_set_text(GTK_ENTRY(entry), entrytext);
	g_free(entrytext);
	
	item = gtk_generic_new(choice);
	gtk_object_set_data(item, GNOME_LINUXCONF_ROW, GINT_TO_POINTER(row));
	handler = gtk_signal_connect(choice,"destroy",
				     GTK_SIGNAL_FUNC(destroy_object),item);
	gtk_object_set_data(item,GNOME_LINUXCONF_DESTROY_HANDLER,
			    GUINT_TO_POINTER(handler));
	gtk_signal_connect(item,"destroy",GTK_SIGNAL_FUNC(choice_destroy_item),
							  NULL);
	gtk_signal_connect(item, "destroy", GTK_SIGNAL_FUNC(free_data), did);
	register_object(item,choiceid,did);

	return item;
}

SETVAL_DEF(Choice)
{
	setval_Combo(args,opts);
}

SETVAL_DEF(Choice_item)
{
	setval_Combo_item(args,opts);
}
