/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static void clistitem_select(GtkCList *clist, gint row, gint column,
	       		     GdkEventButton *event, gpointer data)
{
	char *text = NULL;

	g_return_if_fail(get_object_mainform(GTK_OBJECT(clist)) != NULL);
	text = gtk_clist_get_row_data(clist, row);

	if(text != NULL) {
#ifdef DEBUG
		printf("action %s %s 1\n",
		       get_object_relative_name(GTK_OBJECT(clist)), text);
#endif
		g_print("action %s %s 1\n",
			get_object_relative_name(GTK_OBJECT(clist)), text);
	}

	/* Make sure this widget won't grab forever. (GN) */
	if (GTK_WIDGET_HAS_GRAB (clist)) { 
		gtk_grab_remove (GTK_WIDGET (clist));
		if (gdk_pointer_is_grabbed ())
			gdk_pointer_ungrab (GDK_CURRENT_TIME);
	}
	if (clist->htimer) { 
		gtk_timeout_remove (clist->htimer);
		clist->htimer = 0;
	}
	if (clist->vtimer) {
		gtk_timeout_remove (clist->vtimer);
		clist->vtimer = 0;
	}
}

static void clist_destroy_item(GtkObject *item, gpointer data)
{
	GtkCList *clist = GTK_CLIST(GTK_GENERIC(item)->parent);
	guint dh = GPOINTER_TO_UINT(gtk_object_get_data(item,
					GNOME_LINUXCONF_DESTROY_HANDLER));
	guint sh = GPOINTER_TO_UINT(gtk_object_get_data(GTK_OBJECT(clist),
				        GNOME_LINUXCONF_SELECT_HANDLER));
	int row = clist_find_row(clist, get_object_base_name(item));
	g_return_if_fail(row != -1);
	gtk_signal_handler_block(GTK_OBJECT(clist),sh);
	gtk_clist_remove(clist, row);
	gtk_signal_handler_unblock(GTK_OBJECT(clist),sh);
	gtk_signal_disconnect(GTK_OBJECT(clist), dh);
}

static GtkObject* clist_append_item(GtkCList *clist, const char *clistid,
			     const char *id, GList *list)
{
	char **text = g_malloc0(sizeof(char**) * (GTK_CLIST(clist))->columns);
	GtkObject *item;
	int row, i = 0;
	char *did = g_strdup(id);
	guint handler;
	for(; i < (GTK_CLIST(clist))->columns; i++) {
		if (list != NULL) {
			text[i] = (char*) list->data;
			list = list->next;
		} else {
			break;
		}
	}
	row = gtk_clist_append(GTK_CLIST(clist), text);
	g_free(text);
	gtk_clist_set_row_data(GTK_CLIST(clist), row, did);
	item = gtk_generic_new(GTK_OBJECT(clist));
	gtk_object_set_data(item, GNOME_LINUXCONF_ROW, GINT_TO_POINTER(row));
	/* We need to save the signal handler to disable it when the
	 * item object get destroied (TODO). (GN) */
	handler = gtk_signal_connect(GTK_OBJECT(clist),"destroy",
				     GTK_SIGNAL_FUNC(destroy_object),item);
	gtk_object_set_data(item,GNOME_LINUXCONF_DESTROY_HANDLER,
			    GUINT_TO_POINTER(handler));
	gtk_signal_connect(item,"destroy",GTK_SIGNAL_FUNC(clist_destroy_item),
							  NULL);
	gtk_signal_connect(item,"destroy",GTK_SIGNAL_FUNC(free_data),did);
	gtk_clist_columns_autosize(GTK_CLIST(clist));
	register_object(item,clistid,did);
	return item;
}

int clist_find_row(GtkCList *clist, const char *data)
{
	int i = 0;
	if (data != NULL) {
		for (; i != clist->rows; i++) {
			const char *row_data = gtk_clist_get_row_data(clist,i);
			if (row_data != NULL && strcmp(data,row_data) == 0)
				return i;
		}
	}
	return -1;
}

DUMP_DEF(Clist)
{
}

HANDLE_DEF(Clist)
{
	GtkObject *clist, *sw;
	GtkScrolledWindow *s = NULL;
	char *id = (char*)g_list_nth_data(args, 0);
	char *nbcol = (char*)g_list_nth_data(args, 1);
	int i;
	guint handler;

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(atoi(nbcol) != 0, NULL);

	clist = (GtkObject*) gtk_clist_new(atoi(nbcol));
	gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
	gtk_clist_column_titles_show(GTK_CLIST(clist));
	for(i = 0; i < atoi(nbcol); i++) {
		char *title = (char*)g_list_nth_data(args, 2 + i);
		title = title ? g_strdup(title) : g_strdup("");
		gtk_clist_set_column_title(GTK_CLIST(clist), i, title);
		gtk_signal_connect(clist, "destroy",
				   GTK_SIGNAL_FUNC(free_data), title);
		gtk_clist_set_column_min_width(GTK_CLIST(clist),
			       		       i, i ? 50 : 100);
	}
	gtk_clist_columns_autosize(GTK_CLIST(clist));
	handler = gtk_signal_connect(clist, "select-row",
				     GTK_SIGNAL_FUNC(clistitem_select), NULL);
	gtk_object_set_data(clist,GNOME_LINUXCONF_SELECT_HANDLER,
			    GINT_TO_POINTER(handler));

	sw = (GtkObject*) gtk_scrolled_window_new(NULL, NULL);
	s = GTK_SCROLLED_WINDOW(sw);

	gtk_scrolled_window_set_policy(s,
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_clist_set_hadjustment(GTK_CLIST(clist),
				  gtk_scrolled_window_get_hadjustment(s));
	gtk_clist_set_vadjustment(GTK_CLIST(clist),
				  gtk_scrolled_window_get_vadjustment(s));
	gtk_container_add(GTK_CONTAINER(sw), GTK_WIDGET(clist));
	gtk_signal_connect(clist, "destroy",
			   GTK_SIGNAL_FUNC(destroy_widget), sw);
	gtk_signal_connect(parents_peek(), "dump",
			   GTK_SIGNAL_FUNC(dump_object), clist);

	parent_insert(NULL, sw);
	parents_push(clist);
	register_object_with_parent(clist, id);

	return clist;
}

HANDLE_DEF(Clist_item)
{
	GtkObject *clist;
	GtkObject *item;
	const char *clistid = (const char*)g_list_nth_data(args, 0);
	char *id = (char*)g_list_nth_data(args, 1);

	g_return_val_if_fail(clistid != NULL, NULL);
	g_return_val_if_fail(id != NULL, NULL);
	
	if (strlen(clistid) == 0) {
		clist = parents_peek();
		clistid = get_object_name(clist);
	} else {
		clist = get_object_by_name(clistid);
	}

	g_return_val_if_fail(GTK_IS_CLIST(clist), NULL);
	item = clist_append_item(GTK_CLIST(clist), clistid, id,
				 g_list_next(g_list_next(args)));

	return item;
}

SETVAL_DEF(Clist_item)
{
        const char *pid = (const char*) g_list_nth_data(args, 0);
        const char *id = (const char*) g_list_nth_data(args, 1);
	int r,c,columns;
	GtkObject *object;
	GtkCList *clist;
	object = get_object_by_name_with_parent(strlen(pid)>0?pid:NULL,id);
	g_return_if_fail(GTK_IS_GENERIC(object));
	g_return_if_fail(GTK_IS_CLIST(GTK_GENERIC(object)->parent));
	clist = GTK_CLIST(GTK_GENERIC(object)->parent);
	r = clist_find_row(clist,get_object_base_name(object));
	c = 0;
	columns = clist->columns;
	for (;c < columns; c++) {
		const char *text = (const char*) g_list_nth_data(args,c+2);
		gtk_clist_set_text(clist,r,c,text);
	}
	gtk_clist_columns_autosize(clist);
}
