/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static void combo_destroy_item(GtkObject *item, gpointer data)
{
	GtkCList *clist = GTK_CLIST(GTK_DOUBLE_COMBO(GTK_GENERIC(item)->parent)->clist);
	GtkEntry *entry = GTK_ENTRY(GTK_DOUBLE_COMBO(GTK_GENERIC(item)->parent)->entry);
	guint dh = GPOINTER_TO_UINT(gtk_object_get_data(item,
					GNOME_LINUXCONF_DESTROY_HANDLER));
	char *entrytext;
	int row = clist_find_row(clist, get_object_base_name(item));
	g_return_if_fail(row != -1);
	entrytext = g_strdup(gtk_entry_get_text(entry));
	gtk_clist_remove(clist, row);
	gtk_entry_set_text(entry,entrytext);
	g_free(entrytext);
	gtk_signal_disconnect(GTK_OBJECT(GTK_GENERIC(item)->parent), dh);
}

DUMP_DEF(Combo)
{
	GtkWidget *entry;
	const char *text;
	const char *cname = get_object_relative_name(dumpee);
       	char *name, *c;

	g_return_if_fail(cname != NULL);
	name = g_strdup(cname);
	c = strrchr(name, '.');

	if(c != NULL) {
		*c++ = '\0';
		entry = (GTK_DOUBLE_COMBO(dumpee))->entry;
		text = gtk_entry_get_text(GTK_ENTRY(entry));
		if((text != NULL) && (strlen(text) != 0)) {
#ifdef DEBUG
			printf("dump %s %s %s\n", name, c, text);
#endif
			g_print("dump %s %s %s\n", name, c, text);
		}
	}

	g_free(name);
}

HANDLE_DEF(Combo)
{
	GtkWidget *entry;
	GtkObject *combo;
	char *id = (char*)g_list_nth_data(args, 0);
	/* Not used. (GN) */
	/* char *maxlen = (char*)g_list_nth_data(args, 1); */
	char *val = (char*)g_list_nth_data(args, 2);

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(val != NULL, NULL);
	
	combo = (GtkObject*) gtk_double_combo_new();
	gtk_double_combo_disable_activate(GTK_DOUBLE_COMBO(combo));

	entry = (GTK_DOUBLE_COMBO(combo))->entry;
	gtk_entry_set_text(GTK_ENTRY(entry), val);

	val = g_strdup(val);
	gtk_object_set_data(GTK_OBJECT(entry), GNOME_LINUXCONF_COMBO_DEFAULT,
			    val);
	gtk_signal_connect(GTK_OBJECT(entry), "destroy",
			   GTK_SIGNAL_FUNC(free_data), val);
	
	gtk_object_set_data(combo, GNOME_LINUXCONF_HINT_VFLAGS,
			    GINT_TO_POINTER(GTK_FILL));
	gtk_signal_connect(parents_peek(), "dump",
			   GTK_SIGNAL_FUNC(dump_object), combo);
	
	parent_insert(NULL, combo);
	parents_push(combo);
	register_object_with_parent(combo, id);

	return combo;
}

HANDLE_DEF(Combo_item)
{
	GtkWidget *entry;
	GtkObject *combo, *item;
	GtkCList *clist;
	const char *comboid = (char*)g_list_nth_data(args,0);
	char *id = (char*)g_list_nth_data(args, 1);
	char *val = (char*)g_list_nth_data(args, 2);
	char *comments = concat_strings(g_list_next(g_list_next(g_list_next(args))));
	char *entrytext;
	char *array[2];
	char *did;
	int row;
	uint handler;

	g_return_val_if_fail(comboid != NULL, NULL);
	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(val != NULL, NULL);

	if (strlen(comboid) == 0) {
		combo = parents_peek();
		comboid = get_object_name(combo);
	} else {
		combo = get_object_by_name(comboid);
	}
	
	g_return_val_if_fail(GTK_IS_DOUBLE_COMBO(combo), NULL);

	entry = (GTK_DOUBLE_COMBO(combo))->entry;
	entrytext = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

	array[0] = val;
	array[1] = comments;
	did = g_strdup(id);
	clist = GTK_CLIST(GTK_DOUBLE_COMBO(combo)->clist);
	row = gtk_clist_append(clist, array);
	gtk_clist_set_row_data(clist, row, did);
	gtk_clist_columns_autosize(clist);
	gtk_entry_set_text(GTK_ENTRY(entry), entrytext);
	g_free(entrytext);
	
	item = gtk_generic_new(combo);
	gtk_object_set_data(item, GNOME_LINUXCONF_ROW, GINT_TO_POINTER(row));
	handler = gtk_signal_connect(combo,"destroy",
				     GTK_SIGNAL_FUNC(destroy_object),item);
	gtk_object_set_data(item,GNOME_LINUXCONF_DESTROY_HANDLER,
			    GUINT_TO_POINTER(handler));
	gtk_signal_connect(item,"destroy",GTK_SIGNAL_FUNC(combo_destroy_item),
							  NULL);
	gtk_signal_connect(item, "destroy", GTK_SIGNAL_FUNC(free_data), did);
	register_object(item,comboid,did);

	return item;
}

SETVAL_DEF(Combo_item)
{
        const char *pid = (const char*) g_list_nth_data(args, 0);
        const char *id = (const char*) g_list_nth_data(args, 1);
	GtkObject *object;
	int r, c, columns;
	GtkDoubleCombo *combo;
	GtkCList *clist;
	object = get_object_by_name_with_parent(strlen(pid)>0?pid:NULL,id);
	g_return_if_fail(GTK_IS_GENERIC(object));
	combo = GTK_DOUBLE_COMBO(GTK_GENERIC(object)->parent);
	clist = GTK_CLIST(combo->clist);
	r = clist_find_row(clist,get_object_base_name(object));
	c = 0;
	columns = clist->columns;
	for (;c < columns; c++) {
		const char *text = (const char*) g_list_nth_data(args,c+2);
		gtk_clist_set_text(clist,r,c,text);
	}
	gtk_clist_columns_autosize(clist);
}

SETVAL_DEF(Combo)
{
        const char *pid = (const char*) g_list_nth_data(args, 0);
        const char *id = (const char*) g_list_nth_data(args, 1);
	const char *text = (const char*) g_list_nth_data(args, 2);
	GtkObject *object;
	g_return_if_fail(text != NULL);
	object = get_object_by_name_with_parent(strlen(pid)>0?pid:NULL,id);
	g_return_if_fail(GTK_IS_DOUBLE_COMBO(object));
	gtk_entry_set_text(GTK_ENTRY(GTK_DOUBLE_COMBO(object)->entry), text);
}
