/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static void grab_generic_focus(GtkObject *object)
{
	int type = get_object_type(object);

	if (type == TYPE_CLIST_ITEM) {
		GtkObject *clist = GTK_GENERIC(object)->parent;
		int row = GPOINTER_TO_INT(gtk_object_get_data(object,
					  GNOME_LINUXCONF_ROW));
		guint h = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(clist),
					GNOME_LINUXCONF_SELECT_HANDLER));
		gtk_signal_handler_block(GTK_OBJECT(clist),h);
		gtk_clist_select_row(GTK_CLIST(clist),row,-1);
		gtk_signal_handler_unblock(GTK_OBJECT(clist),h);
	}
}

static void grab_widget_focus(GtkWidget *widget)
{
	if(GTK_IS_NOTEBOOK(widget->parent)) {
		int n = gtk_notebook_page_num(GTK_NOTEBOOK(widget->parent),
					      widget);
		gtk_notebook_set_page(GTK_NOTEBOOK(widget->parent), n);
	} else {
		gtk_widget_grab_focus(widget);
	}
}

/*
static void old_grab_focus(GtkObject *object)
{
	GtkWidget *widget, toplevel;
	int type = get_object_type(object);

	if (type == TYPE_CLIST_ITEM) {
		GtkObject *clist = GTK_GENERIC(object)->parent;
		int row = GPOINTER_TO_INT(gtk_object_get_data(object,
					  GNOME_LINUXCONF_ROW));
		guint h = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(clist),
							"select-row-handler"));
		gtk_signal_handler_block(GTK_OBJECT(clist),h);
		gtk_clist_select_row(GTK_CLIST(clist),row,-1);
		gtk_signal_handler_unblock(GTK_OBJECT(clist),h);
		widget = GTK_WIDGET(clist);
	} else {
		widget = GTK_WIDGET(object);
	}

	if(widget == gtk_widget_get_toplevel(widget)) {
		return;
	}

	if(GTK_IS_WIDGET(widget->parent)) {
		grab_focus(GTK_OBJECT(widget->parent));
	}

	if(GTK_IS_NOTEBOOK(widget->parent)) {
		int n = gtk_notebook_page_num(GTK_NOTEBOOK(widget->parent),
					      widget);
		gtk_notebook_set_page(GTK_NOTEBOOK(widget->parent), n);
	} else {
		gtk_widget_grab_focus(widget);
	}
}
*/

static void grab_focus_recursive(GtkObject *object, GtkObject *toplevel)
{
	if (object != toplevel) {
		GtkObject *parent;
		if (GTK_IS_WIDGET(object)) {
			parent = (GtkObject*) GTK_WIDGET(object)->parent;
			grab_focus_recursive(parent,toplevel);
			grab_widget_focus((GtkWidget*)object);
		} else if (GTK_IS_GENERIC(object)) {
			parent = (GtkObject*) GTK_GENERIC(object)->parent;
			grab_focus_recursive(parent,toplevel);
			grab_generic_focus(object);
		}
	}
}

static void grab_focus(GtkObject *object)
{
	GtkObject *toplevel = object;
	while (GTK_IS_GENERIC(toplevel)) {
		toplevel = GTK_GENERIC(object)->parent;
	}
	if (GTK_IS_WIDGET(toplevel)) {
		toplevel = (GtkObject*) gtk_widget_get_toplevel(GTK_WIDGET(toplevel));
	}
	grab_focus_recursive(object,toplevel);
}

HANDLE_DEF(Curfield)
{
        const char *form = (char*)g_list_nth_data(args, 0);
        char *id = (char*)g_list_nth_data(args, 1);
	char *cid;

	GtkObject *object = NULL;
	
	g_return_val_if_fail(form != NULL, NULL);
	g_return_val_if_fail(id != NULL, NULL);

	/* If it's an empty string, then use the current parent. */
	if (strlen(form) == 0) {
		form = get_object_name(parents_peek());
	}
	cid = g_strdup_printf("%s.%s", form, id);
	object = get_object_by_name(cid);

	if(GTK_IS_OBJECT(object)) {
		grab_focus(object);
	} else {
		g_warning("Curfield: object \"%s\" not found.\n", cid);
	}
	g_free(cid);

	return NULL;
}
