/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

/* Change the number of horizontal and vertical cells used by the last
 * widget and/or realign it */

HANDLE_DEF(Dispolast)
{
	GtkWidget *widget, *parent;
	GtkArg child_args[4];
	GtkArgInfo *child_info[4];

	char *horid = (char*)g_list_nth_data(args, 0);
	char *nbhcell = (char*)g_list_nth_data(args, 1);
	char *vertid = (char*)g_list_nth_data(args, 2);
	char *nbvcell = (char*)g_list_nth_data(args, 3);

	float halign = -1, valign = -1;

	g_return_val_if_fail(horid != NULL, NULL);
	g_return_val_if_fail(nbhcell != NULL, NULL);
	g_return_val_if_fail(vertid != NULL, NULL);
	g_return_val_if_fail(nbvcell != NULL, NULL);

       	widget = (GtkWidget*) previous_object;
	g_assert(GTK_IS_WIDGET(widget));
	
	switch(horid[0]) {
		case 'c': halign = 0.5; break;
		case 'l': halign = 0.0; break;
		case 'r': halign = 1.0; break;
		default:  halign =  -1; break;
	}
	switch(vertid[0]) {
		case 'c': valign = 0.5; break;
		case 't': valign = 0.0; break;
		case 'b': valign = 1.0; break;
		default:  valign =  -1; break;
	}

	if (GTK_IS_SCROLLED_WINDOW(widget->parent))
		widget = widget->parent;
	
	parent = widget->parent;

	/* Verify how to change the alignment. (GN) */
	if(GTK_IS_MISC(widget)) 
		gtk_misc_set_alignment(GTK_MISC(widget),halign,valign);
	else if (GTK_IS_ALIGNMENT(widget)) {
		gpointer hflags, vflags;
		int vexpand = 1, hexpand = 1;
		hflags = gtk_object_get_data(GTK_OBJECT(widget),
					     GNOME_LINUXCONF_HINT_HFLAGS);
		vflags = gtk_object_get_data(GTK_OBJECT(widget),
					     GNOME_LINUXCONF_HINT_VFLAGS);
		if (hflags != NULL && ((GPOINTER_TO_INT(hflags)&GTK_EXPAND)==0))
			hexpand = 0;
		if (vflags != NULL && ((GPOINTER_TO_INT(vflags)&GTK_EXPAND)==0))
			vexpand = 0;
		gtk_alignment_set(GTK_ALIGNMENT(widget),halign,valign,
				  hexpand,vexpand);
	}

	/* If it's 1x1, we only have to realign it. (GN) */
	if((atoi(nbhcell) == 1) && (atoi(nbvcell) == 1)) {
		return NULL;
	}

	if (!GTK_IS_TABLE(parent)) {
		printf("Dispolast: '%s' is not in a table.\n", get_object_name(GTK_OBJECT(widget)));
		printf("Dispolast: its parent is a '%s'.\n", gtk_widget_get_name(parent));
		return NULL;
	}
	//g_return_val_if_fail(GTK_IS_TABLE(parent), NULL);

	gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_TABLE_WIDTH,
			    GINT_TO_POINTER(atoi(nbhcell)));
	gtk_object_set_data(GTK_OBJECT(widget), GNOME_LINUXCONF_TABLE_HEIGHT,
			    GINT_TO_POINTER(atoi(nbvcell)));

	child_args[0].name = "GtkTable::left_attach";
	child_args[1].name = "GtkTable::right_attach";
	child_args[2].name = "GtkTable::top_attach";
	child_args[3].name = "GtkTable::bottom_attach";
	
	gtk_container_child_arg_get_info(gtk_table_get_type(),
					 child_args[0].name,
					 &child_info[0]);
	gtk_container_child_arg_get_info(gtk_table_get_type(),
					 child_args[1].name,
					 &child_info[1]);
	gtk_container_child_arg_get_info(gtk_table_get_type(),
		       			 child_args[2].name,
					 &child_info[2]);
	gtk_container_child_arg_get_info(gtk_table_get_type(),
		       			 child_args[3].name,
					 &child_info[3]);
	
	gtk_container_arg_get(GTK_CONTAINER(parent), widget,
		       	      &child_args[0], child_info[0]);
	gtk_container_arg_get(GTK_CONTAINER(parent), widget,
		       	      &child_args[2], child_info[2]);
	
	/* Change the type to avoid warnings when setting. (GN) */
	child_args[1].type = child_info[1]->type;
	child_args[3].type = child_info[3]->type;
	
	/* Calculate and set the new values. (GN) */
	GTK_VALUE_UINT(child_args[1]) = GTK_VALUE_UINT(child_args[0])
		+ atoi(nbhcell);
	GTK_VALUE_UINT(child_args[3]) = GTK_VALUE_UINT(child_args[2])
		+ atoi(nbvcell);
	
	gtk_container_arg_set(GTK_CONTAINER(parent), widget,
		       	      &child_args[1], child_info[1]);
	gtk_container_arg_set(GTK_CONTAINER(parent), widget,
		       	      &child_args[3], child_info[3]);
	
	/* Set the column where we must put the next widget. (GN) */
	gtk_object_set_data(GTK_OBJECT(parent), GNOME_LINUXCONF_CURRENT_COLUMN,
			    GINT_TO_POINTER(GTK_VALUE_UINT(child_args[1])));
	
	return NULL;
}
