/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

void dump_object(GtkObject *o, gpointer data)
{
	GtkObject *object = (GtkObject*) data;
	int type;
	command_dump_fn *fn;
	g_return_if_fail(GTK_IS_OBJECT(object));
	type = get_object_type(object);
	if(type != -1) {
		fn = command_dumper_by_number(type);
		if(fn == NULL) {
#ifdef DEBUG
			g_warning("No dumper found for a \"%s\".\n", commands[type].name);
#endif
		} else {
#ifdef DEBUG
			printf("Dumping a \"%s\":\n", commands[type].name);
#endif
			fn(object, NULL);
		}
		gtk_signal_emit_by_name(object,"dump");
	}
}

/* Dump out the contents of a widget. */
void dump_widget(GtkWidget *dumpee, gpointer data)
{
	command_dump_fn *fn = NULL;
	int type = get_object_type(GTK_OBJECT(dumpee));
	
	if(type != -1) {
		fn = command_dumper_by_number(type);
		if(fn == NULL) {
			g_warning("Dump: no dumper found for a \"%s\".\n", commands[type].name);
		} else {
#ifdef DEBUG
			printf("Dumping a \"%s\":\n", commands[type].name);
#endif
			fn(GTK_OBJECT(dumpee), data);
		}
	}

	if(GTK_IS_CONTAINER(dumpee)) {
		gtk_container_foreach(GTK_CONTAINER(dumpee),
				      dump_widget, data);
	}
}

HANDLE_DEF(Dump)
{
	char *name = g_list_nth_data(args, 0);
	GtkWidget *dumpee;
       	dumpee = (GtkWidget*) get_object_by_name(name);
	if(dumpee == NULL) {
		g_warning("Dump: widget \"%s\" not found.", name);
	} else {
		dump_widget(dumpee, NULL);
	}
	return NULL;
}
