/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

DUMP_DEF(Form)
{
}

HANDLE_DEF(Form)
{
	GtkWidget *widget, *align, *eventbox;
	char *id = (char*)g_list_nth_data(args, 0);
	const char *vexpands = g_hash_table_lookup(opts,"vexpand");
	const char *hexpands = g_hash_table_lookup(opts,"hexpand");
	const char *hs = g_hash_table_lookup(opts,"h");
	const char *ws = g_hash_table_lookup(opts,"w");
	int hexpand = 1, vexpand = 1;
	int h = -1, w = -1;
	
	g_return_val_if_fail(id != NULL, NULL);

	if (hexpands != NULL) {
		hexpand = atoi(hexpands);
	}
	if (vexpands != NULL) {
		vexpand = atoi(vexpands);
	}

	widget = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(widget), GTK_SHADOW_NONE);

	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(eventbox), widget);
	
	align = gtk_alignment_new(0,0,hexpand,vexpand);
	gtk_container_add(GTK_CONTAINER(align), eventbox);

	gtk_object_set_data(GTK_OBJECT(align), GNOME_LINUXCONF_HINT_VPAD,
			    GINT_TO_POINTER(0));
	gtk_object_set_data(GTK_OBJECT(align), GNOME_LINUXCONF_HINT_VPAD,
			    GINT_TO_POINTER(0));
	
	if (ws != NULL) {
		hexpand = 0;
		w = atoi(ws);
	}
	if (hs != NULL) {
		vexpand = 0;
		h = atoi(hs);
	}

	gtk_widget_set_usize(eventbox, w, h);

	if (vexpand == 0) {
		gtk_object_set_data(GTK_OBJECT(align),
				    GNOME_LINUXCONF_HINT_VFLAGS,
				    GINT_TO_POINTER(GTK_FILL));
	}
	if (hexpand == 0) {
		gtk_object_set_data(GTK_OBJECT(align),
				    GNOME_LINUXCONF_HINT_HFLAGS,
				    GINT_TO_POINTER(GTK_FILL));
	}

	gtk_signal_connect(parents_peek(), "dump",
			   GTK_SIGNAL_FUNC(dump_object), GTK_OBJECT(align));
	gtk_signal_connect(GTK_OBJECT(eventbox), "button_release_event",
			   GTK_SIGNAL_FUNC(inputgrid_event), NULL);
	gtk_signal_connect(GTK_OBJECT(eventbox), "motion_notify_event",
			   GTK_SIGNAL_FUNC(inputgrid_event), NULL);
	gtk_signal_connect(GTK_OBJECT(eventbox), "destroy",
			   GTK_SIGNAL_FUNC(destroy_inputgrid), NULL);

	gtk_object_set_data(GTK_OBJECT(eventbox),
			    GNOME_LINUXCONF_REGISTERED_OBJECT, align);
	gtk_widget_set_events(eventbox, GDK_POINTER_MOTION_MASK);

	parent_insert(NULL, GTK_OBJECT(align));
	parents_push(GTK_OBJECT(align));
	register_object_with_parent(GTK_OBJECT(align), id);

	return GTK_OBJECT(align);
}
