/*
 * A GTK+/GNOME front end for LinuxConf.
 *
 * Copyright (C) 2000 Red Hat Software, Inc.
 *
 * Author: Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#ifndef gnome_linuxconf_h
#define gnome_linuxconf_h

#include <gnome.h>
#include <linuxconf/proto.h>
#include <popt.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <limits.h>
#include <dlfcn.h>
#include <ctype.h>
#include <math.h>

#define HANDLE(x) handle_##x
#define HANDLE_DEF(x) GtkObject *handle_##x(GList *args, GHashTable *opts); \
GtkObject *handle_##x(GList *args, GHashTable *opts)
#define HANDLE_DEC(x) GtkObject *handle_##x(GList *args, GHashTable *opts);
typedef GtkObject *(command_handle_fn)(GList *args, GHashTable *opts);

#define DUMP(x) dump_##x
#define DUMP_DEF(x) void dump_##x(GtkObject *dumpee, gpointer data); \
void dump_##x(GtkObject *dumpee, gpointer data)
#define DUMP_DEC(x) void dump_##x(GtkObject *dumpee, gpointer data);
typedef void (command_dump_fn)(GtkObject *, gpointer);

#define SETVAL(x) setval_##x
#define SETVAL_DEF(x) void setval_##x(GList *args, GHashTable *opts); \
void setval_##x(GList *args, GHashTable *opts)
#define SETVAL_DEC(x) void setval_##x(GList *args, GHashTable *opts);
typedef void (command_setval_fn)(GList *args, GHashTable *opts);

#include "dump.h"
#include "parent.h"
#include "registries.h"
#include "str.h"
#include "util.h"
#include "type.h"
#include "textarea.h"
#include "clist.h"
#include "gtkdoublecombo.h"
#include "gtkgeneric.h"
#include "dec.h"
#include "inputgrid.h"

#define GNOME_LINUXCONF_TYPE		"gnome_linuxconf_type"
#define GNOME_LINUXCONF_NAME		"gnome_linuxconf_name"
#define GNOME_LINUXCONF_TITLE		"gnome_linuxconf_title"
#define GNOME_LINUXCONF_TABLE		"gnome_linuxconf_table"
#define GNOME_LINUXCONF_CURRENT_ROW	"gnome_linuxconf_table_row"
#define GNOME_LINUXCONF_CURRENT_COLUMN	"gnome_linuxconf_table_column"
#define GNOME_LINUXCONF_ROW		"gnome_linuxconf_row"
#define GNOME_LINUXCONF_COLUMN		"gnome_linuxconf_column"
#define GNOME_LINUXCONF_TABLE_WIDTH	"gnome_linuxconf_table_width"
#define GNOME_LINUXCONF_TABLE_HEIGHT	"gnome_linuxconf_table_height"
#define GNOME_LINUXCONF_DESTROY_HANDLER "gnome_linuxconf_destroy_handler"
#define GNOME_LINUXCONF_SELECT_HANDLER  "gnome_linuxconf_select_handler"
#define GNOME_LINUXCONF_MAINFORM	"gnome_linuxconf_mainform"
#define GNOME_LINUXCONF_RADIO_INSTANCE	"gnome_linuxconf_radio_instance"
#define GNOME_LINUXCONF_RADIO_GROUP	"gnome_linuxconf_radio_group"
#define GNOME_LINUXCONF_CHOICE_DEFAULT	"gnome_linuxconf_choice_default"
#define GNOME_LINUXCONF_CHOICE_VALUE	"gnome_linuxconf_choice_value"
#define GNOME_LINUXCONF_CHOICE_TIPS	"gnome_linuxconf_choice_tip"
#define GNOME_LINUXCONF_COMBO_DEFAULT	"gnome_linuxconf_combo_default"
#define GNOME_LINUXCONF_COMBO_TIPS	"gnome_linuxconf_combo_tip"
#define GNOME_LINUXCONF_LIST_DEFAULT	"gnome_linuxconf_list_default"
#define GNOME_LINUXCONF_XPM_DATA	"gnome_linuxconf_xpm_data"
#define GNOME_LINUXCONF_NEEDS_DUMP	"gnome_linuxconf_needs_dump"
#define GNOME_LINUXCONF_WINDOW_TYPE	"gnome_linuxconf_window_type"
#define GNOME_LINUXCONF_SW		"gnome_linuxconf_scrolled_window"
#define GNOME_LINUXCONF_INPUTGRID 	"gnome-linuxconf-inputgrid"
#define GNOME_LINUXCONF_REGISTERED_OBJECT "gnome-linuxconf-registered-object"

#define GNOME_LINUXCONF_POPUP_DONE_ID	"gnome_linuxconf_popup_done_id"

#define GNOME_LINUXCONF_HINT_HFLAGS	"gnome_linuxconf_hint_hflags"
#define GNOME_LINUXCONF_HINT_VFLAGS	"gnome_linuxconf_hint_vflags"
#define GNOME_LINUXCONF_HINT_HPAD	"gnome_linuxconf_hint_hpad"
#define GNOME_LINUXCONF_HINT_VPAD	"gnome_linuxconf_hint_vpad"

#define GNOME_LINUXCONF_CLIP_SIZE	550

command_handle_fn *command_handler_by_name(const char *name);
command_dump_fn *command_dumper_by_name(const char *name);
command_handle_fn *command_handler_by_number(int id);
command_dump_fn *command_dumper_by_number(int id);

typedef struct {
	const char *name;
	command_handle_fn *handle;
	command_dump_fn *dump;
	command_setval_fn *setval;
} command_struct;

extern command_struct commands[];

/* #define DEBUG */

#endif
