/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

#define GET_REAL_FORM(x) while (GTK_IS_ALIGNMENT(x)) x = GTK_OBJECT(GTK_BIN(x)->child)

typedef struct {
	char *id;
	int x;
	int y;
	int cellwidth;
	int cellheight;
	int nbcellh;
	int nbcellv;
	char track;
	int track_cellx;
	int track_celly;
} inputgrid;

HANDLE_DEF(Inputgrid)
{
	const char *id = (const char*) g_list_nth_data(args, 0);
	const char *x = (const char*) g_list_nth_data(args, 1);
	const char *y = (const char*) g_list_nth_data(args, 2);
	const char *cellwidth = (const char*) g_list_nth_data(args, 3);
	const char *cellheight = (const char*) g_list_nth_data(args, 4);
	const char *nbcellh = (const char*) g_list_nth_data(args, 5);
	const char *nbcellv = (const char*) g_list_nth_data(args, 6);
	const char *track = g_hash_table_lookup(opts,"track");
	inputgrid *ig;
	GList *list;
	GtkObject *parent;
	
	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(x != NULL, NULL);
	g_return_val_if_fail(y != NULL, NULL);
	g_return_val_if_fail(cellwidth != NULL, NULL);
	g_return_val_if_fail(cellheight != NULL, NULL);
	g_return_val_if_fail(nbcellh != NULL, NULL);
	g_return_val_if_fail(nbcellv != NULL, NULL);
	
	ig = (inputgrid*) malloc(sizeof(inputgrid));

	ig->id = strdup(id);
	ig->x = atoi(x);
	ig->y = atoi(y);
	ig->cellwidth = atoi(cellwidth);
	ig->cellheight = atoi(cellheight);
	ig->nbcellh = atoi(nbcellh);
	ig->nbcellv = atoi(nbcellv);
	ig->track = (track != NULL && *track == '1');
	ig->track_cellx = -1;
	ig->track_celly = -1;
	
	parent = parents_peek();
	GET_REAL_FORM(parent);
	
	list = (GList*) gtk_object_get_data(parent, GNOME_LINUXCONF_INPUTGRID);
	list = g_list_prepend(list, ig);
	gtk_object_set_data(parent, GNOME_LINUXCONF_INPUTGRID, list);
	
	return NULL;
}

int inputgrid_event(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	GList *list;
	inputgrid *ig;
	int track_cellx, track_celly;
	int event_x = (int)event->x;
	int event_y = (int)event->y;
	list = gtk_object_get_data(GTK_OBJECT(widget), GNOME_LINUXCONF_INPUTGRID);
	if (list) {
		const char *name, *mainform_name;
		char *parent_name;
		GtkWidget *mainform;
		int bs = 0;

		widget = (GtkWidget*) gtk_object_get_data(GTK_OBJECT(widget), GNOME_LINUXCONF_REGISTERED_OBJECT);
		name = get_object_relative_name(GTK_OBJECT(widget));
		g_return_val_if_fail(name != NULL, TRUE);

		/* Get the name of the mainform we're in. */
		mainform = (GtkWidget*) get_object_mainform(GTK_OBJECT(widget));
		g_return_val_if_fail(GTK_IS_WIDGET(mainform), TRUE);
		mainform_name = get_object_base_name(GTK_OBJECT(mainform));

		/* Get the parent widget's name. */
		parent_name = g_strdup(name);
		if(strrchr(parent_name, '.') != NULL) {
			*(strrchr(parent_name, '.')) = '\0';
		}

		for (; list; list = g_list_next(list)) {
			int inside = 0;
			ig = (inputgrid*) list->data;
			if (event_x >= ig->x && event_x < ig->x+ig->cellwidth*ig->nbcellh
			    && event_y >= ig->y && event_y < ig->y+ig->cellheight*ig->nbcellv) {
				track_cellx = (int)((event_x-ig->x)/ig->cellwidth);
				track_celly = (int)((event_y-ig->y)/ig->cellheight);
				inside = 1;
			} else {
				track_cellx = -1;
				track_celly = -1;
			}
			if (((track_cellx != ig->track_cellx || track_celly != ig->track_celly) && ig->track) || (inside && event->type == GDK_BUTTON_RELEASE)) {

				if (event->type == GDK_BUTTON_RELEASE) {
					switch (event->button) {
						case 1: bs |= 1; break;
						case 2: bs |= 2; break;
						case 3: bs |= 4; break;
					}
				}
				if (event->state&GDK_CONTROL_MASK) bs |= 8;
				if (event->state&GDK_SHIFT_MASK) bs |= 16;
				if (event->state&GDK_MOD1_MASK) bs |= 32;

				/* Tell the back-end we were clicked. */
#ifdef DEBUG
				printf("action %s %s %d %d %d\n", strstr(parent_name, mainform_name), ig->id, bs, event_x, event_y);
#endif
				g_print("action %s %s %d %d %d\n", strstr(parent_name, mainform_name), ig->id, bs, event_x, event_y);

			}
			ig->track_cellx = track_cellx;
			ig->track_celly = track_celly;
		}

		free(parent_name);
	}
	return TRUE;
}

void destroy_inputgrid(GtkWidget *widget, gpointer data)
{
	GList *list = (GList*) gtk_object_get_data(GTK_OBJECT(widget), GNOME_LINUXCONF_INPUTGRID);
	if (list) {
		for (; list; list = g_list_next(list)) {
			inputgrid *ig = (inputgrid*) list->data;
			free(ig->id);
			free(ig);
		}
		g_list_free(list);
	}
}

