/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

/* The number of the pipe to use to talk to Linuxconf. */
int ipipefd = STDIN_FILENO;
int opipefd = STDOUT_FILENO;
int parse_tag = 0;
void *dlhandle = NULL;
int logfd = -1;

/* The main command struct. (GN) */
command_struct commands[] = {
{"Noop",HANDLE(Noop),NULL,NULL},
{"Str",HANDLE(Str),NULL,NULL},
{"Newline",HANDLE(Newline),NULL,NULL},
{"Skip",HANDLE(Skip),NULL,NULL},
{"Hline",HANDLE(Hline),DUMP(Hline),NULL},
{"Dispolast",HANDLE(Dispolast),NULL,NULL},
{"Label",HANDLE(Label),DUMP(Label),SETVAL(Label)},
{"String",HANDLE(String),DUMP(String),SETVAL(String)},
{"Checkbox",HANDLE(Checkbox),DUMP(Checkbox),SETVAL(Checkbox)},
{"Radio",HANDLE(Radio),DUMP(Radio),SETVAL(Radio)},
{"Button",HANDLE(Button),DUMP(Button),SETVAL(Button)},
{"Button_xpm",HANDLE(Button_xpm),DUMP(Button_xpm),SETVAL(Button_xpm)},
{"Button_xpmf",HANDLE(Button_xpmf),DUMP(Button_xpmf),NULL},
{"Choice",HANDLE(Choice),DUMP(Choice),SETVAL(Choice)},
{"Choice_item",HANDLE(Choice_item),NULL,SETVAL(Choice_item)},
{"List",HANDLE(List),DUMP(List),NULL},
{"List_item",HANDLE(List_item),NULL,NULL},
{"Combo",HANDLE(Combo),DUMP(Combo),SETVAL(Combo)},
{"Combo_item",HANDLE(Combo_item),NULL,SETVAL(Combo_item)},
{"Group",HANDLE(Group),DUMP(Group),NULL},
{"Form",HANDLE(Form),DUMP(Form),NULL},
{"Page",HANDLE(Page),DUMP(Page),NULL},
{"Book",HANDLE(Book),DUMP(Book),NULL},
{"Mainform",HANDLE(Mainform),DUMP(Mainform),NULL},
{"End",HANDLE(End),NULL,NULL},
{"Delete",HANDLE(Delete),NULL,NULL},
{"Dump",HANDLE(Dump),NULL,NULL},
{"Icon_xpm",HANDLE(Icon_xpm),DUMP(Icon_xpm),NULL},
{"Xfer_xpm",HANDLE(Xfer_xpm),NULL,NULL},
{"Sidetitle",HANDLE(Sidetitle),NULL,NULL},
{"Fill",HANDLE(Fill),DUMP(Fill),NULL},
{"Formbutton",HANDLE(Formbutton),DUMP(Formbutton),NULL},
{"Groupfit",HANDLE(Groupfit),DUMP(Groupfit),NULL},
{"Setweightlast",HANDLE(Setweightlast),NULL,NULL},
{"Richtext",HANDLE(Richtext),DUMP(Richtext),NULL},
{"Password",HANDLE(Password),DUMP(Password),NULL},
{"Buttonfill",HANDLE(Buttonfill),DUMP(Buttonfill),NULL},
{"Enteraction",HANDLE(Enteraction),NULL,NULL},
{"Curfield",HANDLE(Curfield),NULL,NULL},
{"Version",HANDLE(Version),NULL,NULL},
{"Treemenu",HANDLE(Treemenu),DUMP(Treemenu),NULL},
{"Treeelem",HANDLE(Treeelem),NULL,NULL},
{"Clist",HANDLE(Clist),DUMP(Clist),NULL},
{"Clist_item",HANDLE(Clist_item),NULL,SETVAL(Clist_item)},
{"Html",NULL,NULL,NULL},
{"Treesub",HANDLE(Treesub),NULL,NULL},
{"Sheet",HANDLE(Sheet),DUMP(Sheet),NULL},
{"Sheet_item",HANDLE(Sheet_item),NULL,NULL},
{"Textarea",HANDLE(Textarea),DUMP(Textarea),SETVAL(Textarea)},
{"Slider",HANDLE(Slider),NULL,NULL},
{"Gauge",HANDLE(Gauge),DUMP(Gauge),SETVAL(Gauge)},
{"Setval",HANDLE(Setval),NULL,NULL},
{"Alive",HANDLE(Alive),NULL,NULL},
{"Setcontext",HANDLE(Setcontext),NULL,NULL},
{"Defdc",HANDLE(Defdc),NULL,NULL},
{"Deffont",HANDLE(Deffont),NULL,NULL},
{"Defpen",HANDLE(Defpen),NULL,NULL},
{"Defbrush",HANDLE(Defbrush),NULL,NULL},
{"Drawline",HANDLE(Drawline),NULL,NULL},
{"Drawrect",HANDLE(Drawrect),NULL,NULL},
{"Drawarc",HANDLE(Drawarc),NULL,NULL},
{"Fillrect",HANDLE(Fillrect),NULL,NULL},
{"Fillarc",HANDLE(Fillarc),NULL,NULL},
{"Menubar",HANDLE(Menubar),NULL,NULL},
{"Popupmenu",HANDLE(Popupmenu),NULL,NULL},
{"Submenu",HANDLE(Submenu),NULL,NULL},
{"Menuentry",HANDLE(Menuentry),NULL,NULL},
{"Clear",HANDLE(Clear),NULL,NULL},
{"Drawclip",HANDLE(Drawclip),NULL,NULL},
{"Setcursor",NULL,NULL,NULL},
{"Blit",HANDLE(Blit),NULL,NULL},
{"Drawtext",HANDLE(Drawtext),NULL,NULL},
{"Refresh",HANDLE(Refresh),NULL,NULL},
{"Menutitle",NULL,NULL,NULL},
{"Listen",HANDLE(Listen),NULL,NULL},
{"Prefsubmenu",HANDLE(Prefsubmenu),NULL,NULL},
{"Show",HANDLE(Show),NULL,NULL},
{"Inputgrid",HANDLE(Inputgrid),NULL,NULL},
{"Fillpolygon",NULL,NULL,NULL},
{"Splash",HANDLE(Splash),NULL,NULL},
{"Hidesplash",HANDLE(Hidesplash),NULL,NULL},
};

static int total_commands = sizeof(commands) / sizeof(commands[0]);

/* Return the command number for a given command string. */
static int command_number_by_name(const char *name)
{
	int i;
	for(i = 0; i < total_commands; i++) {
		if(strcmp(commands[i].name, name) == 0) {
			return i;
			break;
		}
	}
	return -1;
}

/* Return the command string for a given command number. (GN) */
static const char *command_name_by_number(int id)
{
	if (id >= total_commands)
		return "No-op";
	return commands[id].name;
}

/* Get the handler function by its name. */
command_handle_fn *command_handler_by_name(const char *name)
{
	command_handle_fn *ret = NULL;
	char *cname = g_strdup_printf("handle_%s", name);
	ret = (command_handle_fn*) dlsym(dlhandle, cname);
	g_free(cname);
	return ret;
}

/* Get the dumper function by its name. */
command_dump_fn *command_dumper_by_name(const char *name)
{
	command_dump_fn *ret = NULL;
	char *cname = g_strdup_printf("dump_%s", name);
	ret = (command_dump_fn*) dlsym(dlhandle, cname);
	g_free(cname);
	return ret;
}

/* Get the handler function by its numeric ID. (GN) */
command_handle_fn *command_handler_by_number(int id)
{
	if (id >= total_commands)
		return NULL;
	return commands[id].handle;
}

/* Get the dumper function by its numeric ID. (GN) */
command_dump_fn *command_dumper_by_number(int id)
{
	if (id >= total_commands)
		return NULL;
	return commands[id].dump;
}

/* Parse (with proper quote-handling) a line of input and dispatch it to
 * the proper handler. */
static void parse_line(GScanner *scanner, const char *line)
{
	int len = strlen(line);
	GTokenType token_type;
	GTokenValue token_value;
	GList *args = NULL;
	GHashTable *opts = g_hash_table_new(g_str_hash,g_str_equal);
	command_handle_fn *handle = NULL;
	const char *command_string = NULL;
	int command_num = -1;
	gboolean posit = TRUE;

	/* Feed this particular line to the scanner. */
	g_scanner_input_text(scanner, line, len);

	/* Get the first token. */
	g_scanner_get_next_token(scanner);
	token_type = g_scanner_cur_token(scanner);
	token_value = g_scanner_cur_value(scanner);

	/* What is it? */
	switch(token_type) {
		case G_TOKEN_STRING: {
			/* A string.  Either a type or a specific number in
			 * string form. */
			command_string = token_value.v_string;
			command_num = atoi(command_string);
			/* It's not a name, so it must be a number. */
			if(command_num > 0) {
				handle = command_handler_by_number(command_num);
				command_string = command_name_by_number(command_num);
			}
			/* If it's a name, get the handler. */
			if(handle == NULL) {
				handle = command_handler_by_name(command_string);
				command_num = command_number_by_name(command_string);
				/* Get a constant string. (GN) */
				command_string = command_name_by_number(command_num);
			}
			break;
		}
		case G_TOKEN_INT: {
			/* A number.  It *must* be a real handler number. */
			command_num = token_value.v_int;
			command_string = command_name_by_number(command_num);
			handle = command_handler_by_number(command_num);
			break;
		}
		default: {
			break;
		}
	}

	/* Aaaargh! */
	if(handle == NULL) {
		/* Print out the line we got, for debugging purposes. */
		if(token_type == G_TOKEN_STRING) {
			g_warning("parse_line: unknown or unrecognized command \"%s\".\n",
			          token_value.v_string);
		} else {
			g_warning("parse_line: unknown or unrecognized command %ld.\n",
			          token_value.v_int);
		}
		return;
	}

	/* Read the rest of the line. */
	while((token_type != G_TOKEN_EOF) && (token_type != G_TOKEN_NONE)) {
		g_scanner_get_next_token(scanner);
		token_type = g_scanner_cur_token(scanner);
		token_value = g_scanner_cur_value(scanner);

		switch(token_type) {
			/* Concatenate the string onto the argument list. */
			case G_TOKEN_STRING: {
				char *start;
				start = token_value.v_string;
				/* Is it a non-positional parameter? (GN) */
				if (*(token_value.v_string) == '$') {
					posit = FALSE;
					start++;
				}
				/* This way, any parameter can have a '$'. (GN) */
				if (!posit) {
					char *delim,*end;
					delim = start;
					while (*delim != '=' && *delim != 0) delim++;
					end = delim;
					while (*end != 0) end++;
					g_hash_table_insert(opts,
							g_strndup(start,delim-start),
							g_strndup(delim+1,end+1-delim));
				}
				/* If it's not, just add in the positional
				 * argument list. (GN) */
				else {
					args = g_list_append(args,
						g_strdup(token_value.v_string));
				}
				break;
			}
			/* Concatenate the number onto the argument list. */
			case G_TOKEN_INT: {
				args = g_list_append(args,
						g_strdup_printf("%ld",
							token_value.v_int));
				break;
			}
			case G_TOKEN_EOF:
			case G_TOKEN_ERROR:
			case G_TOKEN_NONE: {
				break;
			}
			default: {
#ifdef DEBUG
				printf("Token type %d\n", token_type);
				fflush(NULL);
				break;
#endif
			}
		}
	}

#ifdef DEBUG
	if(command_string != NULL) {
		char *argstring;
		/* Print out the line we got, for debugging purposes. */
		argstring = concat_strings(args);
		printf("%s %s\n", command_string, argstring);
		fflush(NULL);
		g_free(argstring);
	}
#endif

	/* Now actually create a object if we can. */
	if(handle != NULL) {
		GtkObject *object = handle(args,opts);
		if(object != NULL) {
			/* Keep track of what LinuxConf widget type this is. */
			gtk_object_set_data(object,
					    GNOME_LINUXCONF_TYPE,
					    GINT_TO_POINTER(command_num));
			gtk_object_set_data(object,
					    GNOME_LINUXCONF_MAINFORM,
					    mainforms_peek());
			/* We got the pointer back because we need to register
			 * it, so do so. */
			previous_object = object;
		}
	}

	/* Free the list of arguments we built up previously. */
	if(args != NULL) {
		GList *list;
		for(list = args; list != NULL; list = g_list_next(list)) {
			if(list->data) {
				g_free(list->data);
				list->data = NULL;
			}
		}
		g_list_free(args);
	}

	/* Destroy opts hash table. (GN) */
	g_hash_table_foreach_remove(opts,free_hash_entry,NULL);
	g_hash_table_destroy(opts);

	return;
}

/* Break the input stream up into individual lines and hand them off,
 * one by one, to parse_line. */
static void parse_input(gpointer data, gint source, GdkInputCondition condition)
{
	static char input_buffer[LINE_MAX] = "";
	static unsigned int rest_size = 0;
	char **argv;
	int ret = 0;

	g_return_if_fail(condition == GDK_INPUT_READ);

	ret = read(source,input_buffer+rest_size,
			sizeof(input_buffer)-rest_size-1);
	while(ret != -1) {
		char *rest;

		*(input_buffer+rest_size+ret) = 0;

		/* Log the data. */
		if(logfd != -1) {
			write(logfd, input_buffer, ret);
			fsync(logfd);
		}

		if(ret == 0) {
			/* Note that we can't read/write any more. */
#ifdef DEBUG
			printf("Input pipe closed, done parsing.\n");
#endif
			gdk_input_remove(parse_tag);
			parse_tag = 0;
			close(logfd);
			/* If this was a real pipe, bye-bye. */
			if(ipipefd != STDIN_FILENO) {
				gtk_main_quit();
			}
			break;
		}
		
		/* Terminate the buffer at the last newline, but keep track
		 * of how much data is left over. (GN) */
		rest = rindex(input_buffer, '\n');
		if (rest != NULL){
			*rest++ = 0;
			rest_size = strlen(rest);

			/* Now split the input buffer into lines and feed them
			 * into parse_line. */
			argv = g_strsplit(input_buffer, "\n", sizeof(input_buffer));
			if(argv != NULL) {
				int i;
				for(i = 0; (argv != NULL) && (argv[i] != NULL); i++) {
#ifdef DEBUG
					printf("Parsing line '%s'.\n", argv[i]);
					fflush(stdout);
#endif
					if(strlen(argv[i]) > 0)
						parse_line((GScanner*)data, argv[i]);
				}
				g_strfreev(argv);
				argv = NULL;
			}

			/* Attempt to preserv any partial lines we've read. (GN) */
			if (rest_size > 0)
				memmove(input_buffer, rest, rest_size+1);
		} else {
			rest_size += ret;
		}

		/* Append new data to partial strings. */
		ret = read(source,input_buffer+rest_size,
				sizeof(input_buffer)-rest_size-1);
	}

	/* Add some flicker. */
	/*
	if(parents_peek() != NULL) {
		gtk_widget_show_all(parents_peek());
	}
	*/
}

/* Just print some data to the output pipe. */
static void print_handler(const char *string)
{
	write(opipefd, string, strlen(string));
}

/* Execute a command and return its file descriptors in *in and *out.
 * The original function was part of wxxt-linuxconf. (GN) */
static gboolean exec_cmd(char *command, int *in, int *out)
{
	gboolean ret = FALSE;
	int tbin[2];
	int tbout[2];
	if (pipe(tbin)!=-1 && pipe(tbout)!=-1){
		if (fork()==0){
			char *argv[] = {"sh","-c",command,NULL};
			close (tbin[0]);
			close (tbout[1]);
			dup2 (tbin[1],1);
			dup2 (tbout[0],0);
			execvp (argv[0],argv);
			exit(1);
		}else{
			close (tbin[1]);
			close (tbout[0]);
			*in = tbin[0];
			*out = tbout[1];
			ret = TRUE;
		}
	}
	return ret;
}


int main(int argc, char **argv)
{
	GScannerConfig config;
	GScanner *scanner;
	char *logfile = NULL;
	char *exec = NULL;
	struct poptOption options[] = {
		{"pipe", 'p', POPT_ARG_INT, &ipipefd, 0,
		 "File descriptor to read input from (default 0)", "FD"},
		{"log", 'l', POPT_ARG_STRING, &logfile, 0,
		 "File to log data from linuxconf to", "LOGFILE"},
		{"exec",'e', POPT_ARG_STRING, &exec, 0,
		 "Execute a command and adopt its input/output descriptors", "COMMAND"},
		POPT_AUTOHELP
		{NULL, 0, 0, NULL, 0, NULL, NULL},
	};
	poptContext ctx = NULL;
	char preferences[LINE_MAX] =	"prefer slider gauge reconfdia setval "
					"alive treemenu context listen menu";
	int rc;
#ifdef DEBUG
	putenv ("GTK_MODULES=gle");
#endif
	g_return_val_if_fail((dlhandle = dlopen(NULL, RTLD_NOW)) != NULL, 1);

	/* Try to initialize GNOME. */
	rc = gnome_init_with_popt_table(PACKAGE, VERSION, argc, argv, options,
					0, &ctx);
	if(rc != 0) {
		printf("Couldn't initialize GNOME libraries. Quitting.\n");
		exit(1);
	}

	/* Create the GScanner we use to parse input. */
	memset(&config, 0, sizeof(config));
	config.cset_skip_characters = " \t\n";
	config.cset_identifier_first = G_CSET_A_2_Z G_CSET_a_2_z "0123456789"
				       G_CSET_LATINC G_CSET_LATINS
				       "-=+_|()`'~!@#$%^&*<>[]{}.;:/?,";
	config.cset_identifier_nth = config.cset_identifier_first;
	config.scan_identifier = TRUE;
	config.scan_identifier_1char = TRUE;
	config.scan_string_dq = TRUE;
	config.numbers_2_int = TRUE;
	config.identifier_2_string = TRUE;
	scanner = g_scanner_new(&config);

	/* Parse the command-line arguments. */
	ctx = poptGetContext(PACKAGE, argc, argv, options, 0);
	if(poptGetNextOpt(ctx) != -1) {
		exit(0);
	}

	if (ipipefd != STDIN_FILENO) {
		opipefd = ipipefd;
	}

	/* Open the logfile. */
	if(logfile != NULL) {
		logfd = open(logfile, O_CREAT | O_TRUNC | O_RDWR, 0644);
		g_return_val_if_fail(logfd != -1, 1);
	}

	if(exec != NULL) {
		gboolean exec_ret = exec_cmd(exec, &ipipefd, &opipefd);
		g_return_val_if_fail(exec_ret == TRUE, 1);
	}

	/* Set the input pipe to non-blocking and start listening. */
	fcntl(ipipefd, F_SETFL, O_NONBLOCK);
	parse_tag = gdk_input_add(ipipefd, GDK_INPUT_READ, parse_input, scanner);

	g_set_print_handler(print_handler);

	/* Let the back-end know how we like to do things. */
	g_print("%s\n", preferences);

	/* Add the "dump" user defined signal. (GN) */
	gtk_object_class_user_signal_new (gtk_type_class(GTK_TYPE_OBJECT),
					  "dump",
					  GTK_RUN_LAST|GTK_RUN_ACTION,
					  gtk_marshal_NONE__NONE,
					  GTK_TYPE_NONE, 0);

	/* Let 'er rip. */
	gtk_main();
	return 0;
}
