/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static gboolean action_window_delete(GtkWidget *widget, GdkEvent *event, gpointer data)
{
#ifdef DEBUG
	printf("action %s X\n", (char*) data);
#endif
	g_print("action %s X\n", (char*) data);
	return TRUE;
}

DUMP_DEF(Mainform)
{
}

HANDLE_DEF(Mainform)
{
	GtkWidget *vbox, *widget;
	GtkObject *reconfdia;

	char *id = (char*)g_list_nth_data(args, 0);
	char *title = (char*)g_list_nth_data(args, 1);
	char *type = (char*)g_list_nth_data(args, 2);

	char *rid;

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(title != NULL, NULL);

	if (type == NULL)
		type = "std";
	
	if(GTK_IS_OBJECT(parents_peek())) {
		const char *parent_name = get_object_name(parents_peek());
		rid = g_strdup_printf("%s.%s", parent_name, id);
	} else {
		rid = g_strdup(id);
	}
	reconfdia = get_object_by_name(rid);
	free(rid);
	
	if(reconfdia != NULL) {
		/* Remove the children. */
		gtk_container_foreach(GTK_CONTAINER(reconfdia),
				      (GtkCallback)gtk_widget_destroy, NULL);

		/* In case it held a table, clear the pointers. */
		gtk_object_remove_data(reconfdia, GNOME_LINUXCONF_TABLE);
		gtk_object_remove_data(reconfdia, GNOME_LINUXCONF_SW);

		mainforms_push(reconfdia);
		parents_push(reconfdia);

		return NULL;
	}
	
	if(parents_peek() != NULL) {
		/* Insert as a child of the current parent. NO TITLE FOR YOU! */
		widget = vbox = gtk_vbox_new(FALSE, 0);
		gtk_object_set_data(GTK_OBJECT(widget),
				    GNOME_LINUXCONF_HINT_HFLAGS,
				    GINT_TO_POINTER(GTK_EXPAND | GTK_FILL));
		gtk_object_set_data(GTK_OBJECT(widget),
				    GNOME_LINUXCONF_HINT_VFLAGS,
				    GINT_TO_POINTER(GTK_EXPAND | GTK_FILL));
		gtk_object_set_data(GTK_OBJECT(widget),
			            GNOME_LINUXCONF_HINT_HPAD,
				    GINT_TO_POINTER(-1));
		gtk_object_set_data(GTK_OBJECT(widget),
				    GNOME_LINUXCONF_HINT_VPAD,
				    GINT_TO_POINTER(-1));
	} else {
		/* It's a window.  Construct the right kind of window. */
#if 0
		if(strcasecmp(type, "error") == 0) {
			widget = gtk_window_new(GTK_WINDOW_DIALOG);
		} else if(strcasecmp(type, "notice") == 0) {
			widget = gtk_window_new(GTK_WINDOW_DIALOG);
		} else if(strcasecmp(type, "popup") == 0) {
			widget = gtk_window_new(GTK_WINDOW_DIALOG);
		} else {
			widget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		}
#endif
		/* Linuxconf is building some windows that don't have any
		 * button at all. Some people like to use dialog windows
		 * without bars (no close button), so we must build every
		 * window as a toplevel */
		widget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		window_list_add(widget, NULL);

		gtk_window_set_title(GTK_WINDOW(widget), title);
		gtk_window_set_policy(GTK_WINDOW(widget), FALSE, TRUE, TRUE);

		if((GTK_WINDOW(widget))->type != GTK_WINDOW_TOPLEVEL) {
			gtk_window_set_position(GTK_WINDOW(widget),
						GTK_WIN_POS_CENTER);
			window_list_unbusy_all(NULL, NULL);
		}

		vbox = gtk_vbox_new(FALSE,0);
		gtk_container_add(GTK_CONTAINER(widget),vbox);
		
		gtk_signal_connect(GTK_OBJECT(vbox), "destroy",
				   GTK_SIGNAL_FUNC(destroy_widget), widget);
		gtk_signal_connect(GTK_OBJECT(widget), "destroy",
				   GTK_SIGNAL_FUNC(window_list_remove),
				   NULL);
		id = g_strdup(id);
		gtk_signal_connect(GTK_OBJECT(widget), "delete-event",
				   GTK_SIGNAL_FUNC(action_window_delete), id);
		gtk_signal_connect(GTK_OBJECT(widget), "destroy",
				   GTK_SIGNAL_FUNC(free_data), id);
	}


	title = g_strdup(title);
	gtk_object_set_data(GTK_OBJECT(vbox), GNOME_LINUXCONF_TITLE, title);
	gtk_signal_connect(GTK_OBJECT(vbox), "destroy",
			   GTK_SIGNAL_FUNC(free_data), title);

	if(parents_peek() != NULL) {
		parent_insert(NULL, GTK_OBJECT(widget));
	}
		
	mainforms_push(GTK_OBJECT(vbox));
	parents_push(GTK_OBJECT(vbox));
	register_object_with_parent(GTK_OBJECT(vbox), id);

	return GTK_OBJECT(vbox);
}
