/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static void action_menu(GtkWidget *widget, gpointer data)
{
	GtkObject *form;
	const char *form_name;
	gpointer done_id;

	g_assert(GTK_IS_MENU_ITEM(widget));
	
	/* Disable all windows while Linuxconf processes. */
	window_list_busy_all(NULL, NULL);

	/* Get the name of the mainform we were in. */
	form = get_object_mainform(GTK_OBJECT(widget));
	form_name = get_object_base_name(form);

	done_id = gtk_object_get_data(form, GNOME_LINUXCONF_POPUP_DONE_ID);

	if (done_id != NULL)
		gtk_signal_disconnect(form, GPOINTER_TO_INT(done_id));

	/* Tell the back-end we were clicked. */
#ifdef DEBUG
	printf("menubar %s %s\n", form_name,
	       get_object_base_name(GTK_OBJECT(widget)));
#endif
	g_print("menubar %s %s\n", form_name,
		get_object_base_name(GTK_OBJECT(widget)));
}

static void action_popup_done(GtkWidget *widget, gpointer data)
{
#ifdef DEBUG
	printf("menubar %s -1\n", (char*) data);
#endif
	g_print("menubar %s -1\n", (char*) data);
}

HANDLE_DEF(Menubar)
{
	GtkWidget *widget, *handlebox, *mainform;
	handlebox = gtk_handle_box_new();
	widget = gtk_menu_bar_new();
	gtk_container_add(GTK_CONTAINER(handlebox),widget);
	gtk_widget_show(handlebox);
	gtk_object_set_data(GTK_OBJECT(handlebox), GNOME_LINUXCONF_HINT_VPAD,
			    GINT_TO_POINTER(-1));
	gtk_object_set_data(GTK_OBJECT(handlebox), GNOME_LINUXCONF_HINT_HPAD,
			    GINT_TO_POINTER(-1));
	gtk_object_set_data(GTK_OBJECT(handlebox), GNOME_LINUXCONF_HINT_VFLAGS,
			    GINT_TO_POINTER(GTK_FILL));
	mainform = (GtkWidget*) mainforms_peek();
	gtk_box_pack_start(GTK_BOX(mainform),handlebox,FALSE,TRUE,0);
	parents_push(GTK_OBJECT(handlebox));
	return GTK_OBJECT(handlebox);
}

HANDLE_DEF(Submenu)
{
	GtkWidget *widget, *parent, *item;
	char *title = (char*)g_list_nth_data(args, 0);
	const char *right = g_hash_table_lookup(opts,"right");
	
	parent = (GtkWidget*) parents_peek();

	if (GTK_IS_HANDLE_BOX(parent)) {
		parent = GTK_BIN(parent)->child;
	}
	
	g_return_val_if_fail(GTK_IS_MENU_BAR(parent) || GTK_IS_MENU(parent), NULL);
	g_return_val_if_fail(title != NULL, NULL);
	
	widget = gtk_menu_new();
	item = gtk_menu_item_new_with_label(title);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item),widget);
	
	/* Here we handle the insertion routine locally. (GN) */
	if (GTK_IS_MENU_BAR(parent)) {
		gtk_menu_bar_append(GTK_MENU_BAR(parent),item);
		if (right != NULL && atoi(right) == 1)
			gtk_menu_item_right_justify(GTK_MENU_ITEM(item));
	} else {
		gtk_menu_append(GTK_MENU(parent),item);
	}

	parents_push(GTK_OBJECT(widget));
	
	return GTK_OBJECT(widget);
}

HANDLE_DEF(Menuentry)
{
	GtkWidget *widget, *parent;
	char *id = (char*)g_list_nth_data(args, 0);
	char *title = (char*)g_list_nth_data(args, 1);
	
	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(title != NULL, NULL);
	
	parent = (GtkWidget*) parents_peek();
	g_return_val_if_fail(GTK_IS_MENU(parent), NULL);

	if (strcmp(title, "-") == 0)
		widget = gtk_menu_item_new();
	else
		widget = gtk_menu_item_new_with_label(title);
	
	/* Here we handle the insertion routine locally. (GN) */
	gtk_menu_append(GTK_MENU(parent), widget);

	gtk_signal_connect(GTK_OBJECT(widget), "activate",
			   GTK_SIGNAL_FUNC(action_menu), NULL);

	register_object_with_parent(GTK_OBJECT(widget), id);
	
	return GTK_OBJECT(widget);
}

HANDLE_DEF(Popupmenu)
{
	GtkWidget *widget;
	char *id = (char*)g_list_nth_data(args, 0);
	/* Not used yet. (GN) */
	/* char *title = (char*)g_list_nth_data(args, 1); */
	int done_id;
	widget = gtk_menu_new();
	parents_push(GTK_OBJECT(widget));
	/* Ok, it's not a mainform. But we need to insert it so it can be
	 * referenced when an entry is clicked. (GN) */
	mainforms_push(GTK_OBJECT(widget));
	register_object_with_parent(GTK_OBJECT(widget), id);
	id = g_strdup(id);
	done_id = gtk_signal_connect(GTK_OBJECT(widget), "selection-done",
				     GTK_SIGNAL_FUNC(action_popup_done), id);
	gtk_signal_connect(GTK_OBJECT(widget), "selection-done",
			   GTK_SIGNAL_FUNC(free_data), id);
	gtk_object_set_data(GTK_OBJECT(widget),
			    GNOME_LINUXCONF_POPUP_DONE_ID,
			    GINT_TO_POINTER(done_id));
	return GTK_OBJECT(widget);
}

HANDLE_DEF(Prefsubmenu)
{
	GtkWidget *widget, *parent, *item, *none;
	const char *right = g_hash_table_lookup(opts,"right");
	
	parent = (GtkWidget*) parents_peek();

	if (GTK_IS_HANDLE_BOX(parent)) {
		parent = GTK_BIN(parent)->child;
	}
	
	g_return_val_if_fail(GTK_IS_MENU_BAR(parent) || GTK_IS_MENU(parent), NULL);
	
	//widget = gtk_menu_new();
	item = gtk_menu_item_new_with_label("Preferences");
	//gtk_menu_item_set_submenu(GTK_MENU_ITEM(item),widget);
	
	/* Here we handle the insertion routine locally. (GN) */
	if (GTK_IS_MENU_BAR(parent)) {
		gtk_menu_bar_append(GTK_MENU_BAR(parent),item);
		if (right != NULL && atoi(right) == 1)
			gtk_menu_item_right_justify(GTK_MENU_ITEM(item));
	} else {
		gtk_menu_append(GTK_MENU(parent),item);
	}
	
	//return GTK_OBJECT(widget);
	return GTK_OBJECT(item);
}
