/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

static GtkRadioButton *get_radio(GSList *group, const char *id)
{
	while (group != NULL) {
		GtkObject *radio = (GtkObject*) group->data;
		const char *instance = (const char *) gtk_object_get_data(radio,GNOME_LINUXCONF_RADIO_INSTANCE);
		if (strcmp(instance,id) == 0)
			return (GtkRadioButton*) radio;
		group = g_slist_next(group);
	}
	return NULL;
}

static const char *get_active_radio(GSList *group)
{
	while (group != NULL) {
		GtkObject *radio = (GtkObject*) group->data;
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio)))
			return (const char *) gtk_object_get_data(radio,GNOME_LINUXCONF_RADIO_INSTANCE);
		group = g_slist_next(group);
	}
	return NULL;
}

DUMP_DEF(Radio)
{
	const char *rid;
	char *rpid, *id;
	GSList *group;
	const char *selected;
	
	g_return_if_fail(GTK_IS_OBJECT(dumpee));
	rid = get_object_relative_name(dumpee);
	
	g_return_if_fail(rid != NULL);
	group = gtk_object_get_data(dumpee, GNOME_LINUXCONF_RADIO_GROUP);

	rpid = g_strdup(rid);
	id = strrchr(rpid, '.');
	if (id != NULL) {
		*id++ = '\0';
	}

	selected = get_active_radio(group);
	if (selected != NULL) {
#ifdef DEBUG
		printf("dump %s %s %s\n", rpid, id, selected);
#endif
		g_print("dump %s %s %s\n", rpid, id, selected);
	}
	g_free(rpid);
}

HANDLE_DEF(Radio)
{
	GSList *group = NULL;
	GtkObject *radio,*object,*align;

	char *id = (char*)g_list_nth_data(args, 0);
	char *instance = (char*)g_list_nth_data(args, 1);
	char *state = (char*)g_list_nth_data(args, 2);
	char *text = (char*)g_list_nth_data(args, 3);
	char *dinstance;

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(instance != NULL, NULL);
	g_return_val_if_fail(state != NULL, NULL);

	/* Get the object that will handle the whole group. (GN) */
	object = get_object_by_name_with_parent(NULL,id);
	if (object == NULL) {
		object = gtk_generic_new(parents_peek());
		gtk_signal_connect(parents_peek(), "dump",
				   GTK_SIGNAL_FUNC(dump_object), object);
	} else {
		group = (GSList*) gtk_object_get_data(object,GNOME_LINUXCONF_RADIO_GROUP);
	}
	
	if (text == NULL) {
		radio = (GtkObject*) gtk_radio_button_new(group);
	} else {
		radio = (GtkObject*) gtk_radio_button_new_with_label(group, text);
	}
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio));
	gtk_object_set_data(object,GNOME_LINUXCONF_RADIO_GROUP,group);

	dinstance = g_strdup(instance);
	gtk_object_set_data(radio, GNOME_LINUXCONF_RADIO_INSTANCE, dinstance);
	gtk_signal_connect(radio, "destroy",
			   GTK_SIGNAL_FUNC(free_data), dinstance);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), atoi(state));

	/* Add an alignment since it can't handle it. (GN) */
	align = (GtkObject*) gtk_alignment_new(0,0.5,0,0);
	gtk_container_add(GTK_CONTAINER(align),GTK_WIDGET(radio));
	gtk_object_set_data(align, GNOME_LINUXCONF_HINT_VFLAGS,
			    GINT_TO_POINTER(GTK_FILL));
	gtk_object_set_data(align, GNOME_LINUXCONF_HINT_HFLAGS,
			    GINT_TO_POINTER(GTK_FILL));

	gtk_signal_connect(object, "destroy",
			   GTK_SIGNAL_FUNC(destroy_widget), align);
	gtk_signal_connect(parents_peek(), "destroy",
			   GTK_SIGNAL_FUNC(destroy_object), object);
	parent_insert(NULL, align);
	register_object_with_parent(object, id);
	
	return object;
}

SETVAL_DEF(Radio)
{
        const char *pid = (const char*) g_list_nth_data(args, 0);
        const char *id = (const char*) g_list_nth_data(args, 1);
	const char *instance = (const char*) g_list_nth_data(args, 2);
	GtkObject *object;
	GtkRadioButton *radio;
	GSList *group;
	g_return_if_fail(instance != NULL);
	object = get_object_by_name_with_parent(strlen(pid)>0?pid:NULL,id);
	g_return_if_fail(GTK_IS_GENERIC(object));
	group = gtk_object_get_data(object,GNOME_LINUXCONF_RADIO_GROUP);
	g_return_if_fail(group != NULL);
	radio = get_radio(group,instance);
	g_return_if_fail(GTK_IS_RADIO_BUTTON(radio));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);
}
