/*
 * A GTK+/GNOME front end for Linuxconf.
 *
 * Copyright (C) 2000, 2001 Conectiva, Inc.
 * Copyright (C) 2000 Red Hat, Inc.
 *
 * Authors: Gustavo Niemeyer <niemeyer@conectiva.com>
 *          Nalin Dahyabhai <nalin@redhat.com>
 *
 */

#include "gnome-linuxconf.h"

int textarea_position;

DUMP_DEF(Textarea)
{
	const char *cname = get_object_relative_name(dumpee);
	char *path, *c;
	g_return_if_fail(cname != NULL);
	path = g_strdup(cname);
	c = strrchr(path, '.');
	if(c != NULL) {
		gint i, w = gtk_text_get_length(GTK_TEXT(dumpee));
		char *text = gtk_editable_get_chars(GTK_EDITABLE(dumpee), 0, w);
		char **lines = g_strsplit(text, "\n", LINE_MAX);

		*c++ = 0;

		for(i = 0; (lines != NULL) && (lines[i] != NULL); i++) {
#ifdef DEBUG
			printf("dump %s %s %s\n", path, c, lines[i]);
#endif
			g_print("dump %s %s %s\n", path, c, lines[i]);
		}
	}
	g_free(path);
}

HANDLE_DEF(Textarea)
{
	GtkWidget *widget, *sw;
	char *id = (char*)g_list_nth_data(args, 0);
	char *cols = (char*)g_list_nth_data(args, 1);
	char *rows = (char*)g_list_nth_data(args, 2);

	g_return_val_if_fail(id != NULL, NULL);
	g_return_val_if_fail(cols != NULL, NULL);
	g_return_val_if_fail(rows != NULL, NULL);

	widget = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(widget),TRUE);
	/* The text widget doesn't have horizontal scrolling yet. (GN) */
	/* gtk_text_set_line_wrap(GTK_TEXT(widget),FALSE); */
	sw = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				       GTK_POLICY_NEVER,
				       GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(sw), widget);

	gtk_widget_set_usize(sw, atoi(cols)*8, atoi(rows)*14);
	
	textarea_position = 0;

	gtk_signal_connect(parents_peek(), "dump",
			   GTK_SIGNAL_FUNC(dump_object), GTK_OBJECT(widget));
	
	parent_insert(NULL, GTK_OBJECT(sw));
	register_object_with_parent(GTK_OBJECT(widget), id);
	parents_push(GTK_OBJECT(widget));

	return GTK_OBJECT(widget);
}

SETVAL_DEF(Textarea)
{
        const char *pid = (const char*) g_list_nth_data(args, 0);
        const char *id = (const char*) g_list_nth_data(args, 1);
	const char *command = (const char*) g_list_nth_data(args, 2);
	GtkObject *object;
	g_return_if_fail(command != NULL);
	object = get_object_by_name_with_parent(strlen(pid)>0?pid:NULL,id);
	g_return_if_fail(GTK_IS_TEXT(object));
	if (strcmp(command, "reset") == 0) {
		textarea_position = 0;
		gtk_text_set_point(GTK_TEXT(object), 0);
		gtk_text_forward_delete(GTK_TEXT(object), gtk_text_get_length(GTK_TEXT(object)));
		parents_push(object);
	} else {
		g_warning("unknown Textarea Setval command");
	}
}
